@@ -1,8 +1,11 @@
-//! Provides definitions required for the `ascent` macro. Not intended for direct use.
+// Provides definitions required for the `ascent` macro. Not intended for direct use.
+#![doc(hidden)]
 
-use std::{collections::{HashMap, HashSet}, hash::{BuildHasherDefault, Hash}, rc::Rc, time::{Duration, Instant}};
+pub use crate::convert::*;
 
-use ascent_base::Lattice;
+use std::{collections::{HashMap, HashSet}, hash::{BuildHasherDefault, Hash}, time::{Duration, Instant}};
+
+use ascent_base::{Lattice};
 use nohash_hasher::BuildNoHashHasher;
 use rustc_hash::FxHasher;
 
@@ -10,7 +13,9 @@ use rustc_hash::FxHasher;
 pub type RelIndexType<K> = HashMap<K, Vec<usize>, BuildHasherDefault<FxHasher>>;
 pub type LatticeIndexType<K> = HashMap<K, HashSet<usize, BuildNoHashHasher<usize>>, BuildHasherDefault<FxHasher>>;
 // pub type RelFullIndexType<K> = HashMap<K, usize, BuildHasherDefault<FxHasher>>;
-pub type RelFullIndexType<K> = hashbrown::HashMap<K, usize, BuildHasherDefault<FxHasher>>;
+pub(crate) type HashBrownRelFullIndexType<K> = hashbrown::HashMap<K, usize, BuildHasherDefault<FxHasher>>;
+pub type RelFullIndexType<K> = HashBrownRelFullIndexType<K>;
+
 pub type RelNoIndexType = Vec<usize>;
 
 pub trait RelIndexTrait{
@@ -19,23 +24,12 @@ pub trait RelIndexTrait{
    fn index_insert(ind: &mut Self, key: Self::Key, tuple_index: usize);
 }
 
-// pub trait RelIndexIterAllTrait<'a>{
-//    type IteratorType: Iterator<Item = &'a usize>;
-//    fn iter_all(&'a self) -> Self::IteratorType;
-// }
-// impl<'a, K: 'a> RelIndexIterAllTrait<'a> for RelIndexType<K> {
-//    type IteratorType = FlatMap<Values<'a, K, Vec<usize>>, Iter<'a, usize>, fn(& Vec<usize>) -> Iter<usize>>;
-//    // type IteratorType = Iter<'a, usize>;
-//    fn iter_all(&'a self) -> Self::IteratorType {
-//       fn vec_iter(vec: &Vec<usize>) -> Iter<usize> {
-//          vec.iter()
-//       }
-//       let res = self.values().flat_map(vec_iter);
-//       // let res = self.values().next().unwrap().iter();
-//       res
-//       // todo!()
-//    }
-// }
+pub trait RelFullIndexTrait : Default {
+   type Key: Clone;
+   /// if an entry for `key` does not exsit, inserts `v` for it and returns true.
+   fn insert_if_not_present(&mut self, key: &Self::Key, v: usize) -> bool;
+}
+
 
 pub static mut MOVE_REL_INDEX_CONTENTS_TOTAL_TIME : Duration = Duration::ZERO;
 pub static mut INDEX_INSERT_TOTAL_TIME : Duration = Duration::ZERO;
@@ -85,15 +79,15 @@ impl<K: Eq + Hash> RelIndexTrait for RelIndexType<K>{
    }
 }
 impl RelIndexTrait for RelNoIndexType {
-    type Key = ();
+   type Key = ();
 
-    fn move_index_contents(ind1: &mut Self, ind2: &mut Self) {
-        ind2.append(ind1);
-    }
+   fn move_index_contents(ind1: &mut Self, ind2: &mut Self) {
+      ind2.append(ind1);
+   }
 
-    fn index_insert(ind: &mut Self, _key: Self::Key, tuple_index: usize) {
-        ind.push(tuple_index);
-    }
+   fn index_insert(ind: &mut Self, _key: Self::Key, tuple_index: usize) {
+      ind.push(tuple_index);
+   }
 }
 
 impl<K: Eq + Hash> RelIndexTrait for LatticeIndexType<K>{
@@ -114,24 +108,26 @@ impl<K: Eq + Hash> RelIndexTrait for LatticeIndexType<K>{
 }
 
 pub static mut MOVE_FULL_INDEX_CONTENTS_TOTAL_TIME : Duration = Duration::ZERO;
-impl<K: Eq + Hash> RelIndexTrait for RelFullIndexType<K>{
+impl<K: Eq + Hash> RelIndexTrait for HashBrownRelFullIndexType<K>{
     type Key = K;
 
    fn move_index_contents(from: &mut Self, to: &mut Self) {
-      // let before = Instant::now();
+      let before = Instant::now();
       if from.len() > to.len() {
          std::mem::swap(from, to);
       }
       to.reserve(from.len());
       for (k, v) in from.drain() {
+         // to.raw_entry_mut().from_hash(42, |x| x == &k).insert(k, v);
          to.insert(k, v); // TODO could be improved
       }
-      // unsafe {
-      //    MOVE_FULL_INDEX_CONTENTS_TOTAL_TIME += before.elapsed();
-      // }
+      unsafe {
+         MOVE_FULL_INDEX_CONTENTS_TOTAL_TIME += before.elapsed();
+      }
 
    }
 
+   #[inline(always)]
    fn index_insert(hm: &mut Self, key: Self::Key, tuple_index: usize) {
       hm.insert(key, tuple_index);
       // TODO undo this
@@ -140,18 +136,19 @@ impl<K: Eq + Hash> RelIndexTrait for RelFullIndexType<K>{
    }
 }
 
-#[inline]
-/// if an entry for `key` does not exsit, inserts `v` for it and returns true.
-pub fn full_index_insert_if_not_present<K: Eq + Hash + Clone>(hm: &mut RelFullIndexType<K>, key: &K, v: usize) -> bool {
-
-   match hm.raw_entry_mut().from_key(key) {
-    hashbrown::hash_map::RawEntryMut::Occupied(_) => false,
-    hashbrown::hash_map::RawEntryMut::Vacant(vacant) => {vacant.insert(key.clone(), v); true},
+impl <K: Clone + Hash + Eq> RelFullIndexTrait for HashBrownRelFullIndexType<K> {
+   type Key = K;
+   #[inline]
+   fn insert_if_not_present(&mut self, key: &Self::Key, v: usize) -> bool {
+      match self.raw_entry_mut().from_key(key) {
+         hashbrown::hash_map::RawEntryMut::Occupied(_) => false,
+         hashbrown::hash_map::RawEntryMut::Vacant(vacant) => {vacant.insert(key.clone(), v); true},
+      }
+      // match hm.entry(key) {
+      //    Occupied(_) => false,
+      //    Vacant(vacant) => {vacant.insert(v); true},
+      // }
    }
-   // match hm.entry(key) {
-   //    Occupied(_) => false,
-   //    Vacant(vacant) => {vacant.insert(v); true},
-   // }
 }
 
 
@@ -164,32 +161,6 @@ pub struct LatTypeConstraints<T> where T : Clone + Eq + Hash + Lattice{_t: ::cor
 pub fn comment(_: &str){}
 
 
-pub trait Convert<TSource> {
-   fn convert(source: TSource) -> Self;
-}
-
-impl<T> Convert<T> for T {
-   #[inline(always)]
-   fn convert(source: T) -> T {source}
-}
-
-impl<T> Convert<&T> for T where T: Clone {
-   #[inline(always)]
-   fn convert(source: &T) -> T {source.clone()}
-}
-
-impl Convert<&str> for String {
-   fn convert(source: &str) -> Self {
-      source.to_string()
-   }
-}
-
-impl<T: Clone> Convert<&Rc<T>> for T {
-   fn convert(source: &Rc<T>) -> Self {
-      source.as_ref().clone()
-   }
-}
-
 #[inline]
 pub fn rel_ind_val_option_to_iter<'a>(val_option: Option<&'a Vec<usize>>) -> std::slice::Iter<'a, usize> {
    match val_option {
